/*
* Copyright (C) 2013 Invensense, Inc.
*
* This software is licensed under the terms of the GNU General Public
* License version 2, as published by the Free Software Foundation, and
* may be copied, distributed, and modified under those terms.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*/
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/i2c.h>
#include <linux/err.h>
#include <linux/delay.h>
#include <linux/sysfs.h>
#include <linux/jiffies.h>
#include <linux/irq.h>
#include <linux/interrupt.h>
#include <linux/kfifo.h>
#include <linux/poll.h>
#include <linux/miscdevice.h>
#include <linux/spinlock.h>

#include "iio.h"
#include "sysfs.h"
#include "trigger.h"
#include "inv_ak09911_iio.h"

static const struct iio_trigger_ops inv_ak09911_trigger_ops = {
	.owner = THIS_MODULE,
};

int inv_ak09911_probe_trigger(struct iio_dev *indio_dev)
{
	int ret;
	struct inv_ak09911_state_s *st = iio_priv(indio_dev);

#ifdef INV_KERNEL_3_10
	st->trig = iio_trigger_alloc("%s-dev%d",
#else
	st->trig = iio_allocate_trigger("%s-dev%d",
#endif
					indio_dev->name,
					indio_dev->id);
	if (st->trig == NULL) {
		ret = -ENOMEM;
		goto error_ret;
	}
	/* select default trigger */
	st->trig->dev.parent = &st->i2c->dev;
#ifndef INV_KERNEL_3_10
	st->trig->private_data = indio_dev;
#endif
	st->trig->ops = &inv_ak09911_trigger_ops;
	ret = iio_trigger_register(st->trig);

	/* select default trigger */
	indio_dev->trig = st->trig;
	if (ret)
		goto error_free_trig;

	return 0;

error_free_trig:
#ifdef INV_KERNEL_3_10
		iio_trigger_free(st->trig);
#else
		iio_free_trigger(st->trig);
#endif
error_ret:
	return ret;
}

void inv_ak09911_remove_trigger(struct iio_dev *indio_dev)
{
	struct inv_ak09911_state_s *st = iio_priv(indio_dev);

	iio_trigger_unregister(st->trig);
#ifdef INV_KERNEL_3_10
	iio_trigger_free(st->trig);
#else
	iio_free_trigger(st->trig);
#endif
}

