/*************************************************************************/ /*!
@File
@Title          Initialisation server parameters header
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@Description    Defines the connections between the various parts of the
		initialisation server.
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef __SRVINIT_PARAM_H__
#define __SRVINIT_PARAM_H__

#if defined(__linux__) && defined(__KERNEL__)
#include "os_srvinit_param.h"
#else

#if defined (__cplusplus)
extern "C" {
#endif

#include "img_defs.h"
#include "img_types.h"

typedef struct
{
	IMG_CHAR *pszValue;
	IMG_UINT32 ui32Value;
} SRV_INIT_PARAM_UINT32_LOOKUP;

void *SrvInitParamOpen(void);
void SrvInitParamClose(void *pvState);

#define	SrvInitParamInitBOOL(name, defval) \
	static const IMG_BOOL __SrvInitParam_ ## name = defval;

void _SrvInitParamGetBOOL(void *pvState, const IMG_CHAR *pszName, const IMG_BOOL *pbDefault, IMG_BOOL *pbValue);

#define	SrvInitParamGetBOOL(state, name, value) \
		_SrvInitParamGetBOOL(state, # name, & __SrvInitParam_ ## name, &(value))

#define	SrvInitParamInitUINT32(name, defval) \
	static const IMG_UINT32 __SrvInitParam_ ## name = defval;

void _SrvInitParamGetUINT32(void *pvState, const IMG_CHAR *pszName, const IMG_UINT32 *pui32Default, IMG_UINT32 *pui32Value);

#define	SrvInitParamGetUINT32(state, name, value) \
		_SrvInitParamGetUINT32(state, # name, & __SrvInitParam_ ## name, &(value))

#define	SrvInitParamInitUINT32BitField(name, inival, lookup) \
	static IMG_UINT32 __SrvInitParam_ ## name = inival; \
	static SRV_INIT_PARAM_UINT32_LOOKUP * \
		__SrvInitParamLookup_ ## name = &lookup[0]; \
	static const IMG_UINT32 __SrvInitParamSize_ ## name = \
					(sizeof(lookup) / sizeof(lookup[0]));

IMG_BOOL _SrvInitParamGetUINT32BitField(void *pvState, const IMG_CHAR *pszBaseName, IMG_UINT32 uiDefault, const SRV_INIT_PARAM_UINT32_LOOKUP *psLookup, IMG_UINT32 uiSize, IMG_UINT32 *puiValue);

#define	SrvInitParamGetUINT32BitField(state, name, value) \
		_SrvInitParamGetUINT32BitField(state, # name, __SrvInitParam_ ## name, __SrvInitParamLookup_ ## name, __SrvInitParamSize_ ## name, &(value))

#define	SrvInitParamInitUINT32List(name, defval, lookup) \
	static IMG_UINT32 __SrvInitParam_ ## name = defval; \
	static SRV_INIT_PARAM_UINT32_LOOKUP * \
		__SrvInitParamLookup_ ## name = &lookup[0]; \
	static const IMG_UINT32 __SrvInitParamSize_ ## name = \
					(sizeof(lookup) / sizeof(lookup[0]));

IMG_BOOL _SrvInitParamGetUINT32List(void *pvState, const IMG_CHAR *pszName, IMG_UINT32 uiDefault, const SRV_INIT_PARAM_UINT32_LOOKUP *psLookup, IMG_UINT32 uiSize, IMG_UINT32 *puiValue);

#define	SrvInitParamGetUINT32List(state, name, value) \
		_SrvInitParamGetUINT32List(state, # name, __SrvInitParam_ ## name, __SrvInitParamLookup_ ## name, __SrvInitParamSize_ ## name, &(value))

#if defined (__cplusplus)
}
#endif

#endif

#endif /* __SRVINIT_PARAM_H__ */
